/*  
	Extension:	Wikipedia formatting context menu for Mozilla based browsers
	Author: 	Bananeweizen, http://www.bananeweizen.de
	Based on:	bbcode extension of Jed Brown, http://jedbrown.net/
				PastQuote extension by Cusser, http://cusser.net/extensions/
*/

var gPreferences;
var wikipedia_stringBundle;
var wikipedia_Document = null;

//Set Defaults
getPrefwikipediaCompose().SetDefaultBoolPref("wikipedia.contextmenu.wikipedia", true);
wikipedia_enablewikipedia = getPrefwikipediaCompose().GetBoolPref("wikipedia.contextmenu.wikipedia");

var wikipedia = {
	init: function() {
    wikipedia_setupDocument(document);
    try{
      var menu = document.getElementById('contentAreaContextMenu');
      menu.addEventListener('popupshowing', wikipedia.showHide, false);
    }catch (e){
      //Foobar!
    }
	},
	showHide: function() {
      document.getElementById('context-wikipedia').hidden = document.getElementById('context-undo').hidden;
	},
	wikipedia: function(myCommand, forceSelection) {
    var clip = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
    if (!clip){
      return false;
    }
    
    var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
    if (!trans){
      return false;
    }
    
    trans.addDataFlavor("text/unicode");
    clip.getData(trans,clip.kGlobalClipboard);

    var str = null;
    var str_clipboard=new Object();
    var strLength=new Object();
    try{
      trans.getTransferData("text/unicode",str_clipboard,strLength);
  
  
      if (str_clipboard){
        str_clipboard=str_clipboard.value.QueryInterface(Components.interfaces.nsISupportsString);
      }
      if (str_clipboard){
        pastetext=str_clipboard.data.substring(0,strLength.value / 2);
      }
    }catch (e){
      //alert("No text in the clipboard, please copy something first.");
    }
    
		var theBox = document.commandDispatcher.focusedElement;
		var oPosition = theBox.scrollTop;
		var oHeight = theBox.scrollHeight;
    
    
		//Get Selected text (if selected)
		var startPos = theBox.selectionStart;
		var endPos = theBox.selectionEnd;
		str = theBox.value.substring(startPos, endPos);
		
		//check for mandatory input
		if (forceSelection) {
			if (str==null || str=="") {
				str=getInput("wikipedia.input."+myCommand)
				if (str==null || str=="") {
					return;
				}
			}
		}

        wikipedia.insertAtCursorSetup(myCommand, str_clipboard, str, theBox);
		var nHeight = theBox.scrollHeight - oHeight;
		theBox.scrollTop = oPosition + nHeight;
	},
  
  	insertAtCursorSetup: function(myCommand, str_clipboard, str, theBox) {
  // Function taken from http://www.alexking.org/blog/2003/06/02/inserting-at-the-cursor-using-javascript/
  // Modified to return cursor to correct place
    switch (myCommand){
      case "link":
        wikipedia.insertAtCursor(theBox,"[[" + str + "]]");
        break;

      case "bold":
        wikipedia.insertAtCursor(theBox,"'''" + str + "'''");
        break;
        
      case "italic":
        wikipedia.insertAtCursor(theBox,"''" + str + "''");
        break;
        
      case "bolditalic":
        wikipedia.insertAtCursor(theBox,"'''''" + str + "'''''");
        break;
        
      case "underline":
        wikipedia.insertAtCursor(theBox,"<u>" + str + "</u>");
        break;
        
      case "strike":
        wikipedia.insertAtCursor(theBox,"<strike>" + str + "<strike>");
        break;
        
      case "small":
        wikipedia.insertAtCursor(theBox,"<small>" + str + "</small>");
        break;

      case "paragraph":
        wikipedia.insertAtCursor(theBox,"\n==" + str + "==\n");
        break;
        
      case "subparagraph":
        wikipedia.insertAtCursor(theBox,"\n===" + str + "===\n");
        break;
        
      case "subsubparagraph":
        wikipedia.insertAtCursor(theBox,"\n====" + str + "====\n");
        break;
        
      case "weblink":
        wikipedia.insertAtCursor(theBox,"[" + str + "]");
        break;


      case "simplelist":
        wikipedia.insertAtCursor(theBox,"*\n*\n*\n");
        break;

      case "numberedlist":
        wikipedia.insertAtCursor(theBox,"#\n#\n#\n");
        break;

      case "definitionlist":
        var topic=wikipedia_stringBundle.getString("wikipedia.definitionlist.topic");
        var definition=wikipedia_stringBundle.getString("wikipedia.definitionlist.definition");
        var def=";"+topic+":"+definition+"\n";
        wikipedia.insertAtCursor(theBox,def+def);
        break;


      case "nbsp":
        wikipedia.insertAtCursor(theBox,"&nbsp;");
        break;
        
      case "dash":
        wikipedia.insertAtCursor(theBox,"");
        break;
        
      case "auml":
        wikipedia.insertAtCursor(theBox,"");
        break;
        
      case "ouml":
        wikipedia.insertAtCursor(theBox,"");
        break;
        
      case "uuml":
        wikipedia.insertAtCursor(theBox,"");
        break;
        
      case "aumlcap":
        wikipedia.insertAtCursor(theBox,"");
        break;
        
      case "oumlcap":
        wikipedia.insertAtCursor(theBox,"");
        break;
        
      case "uumlcap":
        wikipedia.insertAtCursor(theBox,"");
        break;
        
      case "szlig":
        wikipedia.insertAtCursor(theBox,"");
        break;
        
      case "euro":
        wikipedia.insertAtCursor(theBox,"");
        break;
        
      case "dead":
        wikipedia.insertAtCursor(theBox,"");
        break;
        
		//all other text pieces are just inserted
      default : 
        wikipedia.insertAtCursor(theBox, myCommand);
        
    }
  },

  insertAtCursor: function(theBox, aText) {
    try {
      var command = "cmd_insertText";
      var controller = document.commandDispatcher.getControllerForCommand(command);
      if (controller && controller.isCommandEnabled(command)) {
        controller = controller.QueryInterface(Components.interfaces.nsICommandController);
        var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
        params = params.createInstance(Components.interfaces.nsICommandParams);
        params.setStringValue("state_data", aText);
        controller.doCommandWithParams(command, params);
      }
    }catch (e) {
      dump("Can't do cmd_insertText! ");
      dump(e+"\n")
    }
  }
}

function getInput(textId) {
	var dialogText=wikipedia_stringBundle.getString(textId);
	var newtext = prompt(dialogText);
	return newtext;
}

//http://www.xulplanet.com/references/xpcomref/ifaces/nsIPref.html
function getPrefwikipediaCompose() {
  if (!gPreferences) {
    try {
      gPreferences = Components.classes["@mozilla.org/preferences;1"].getService();
      gPreferences = gPreferences.QueryInterface(
        Components.interfaces.nsIPref);
    } catch (e) {}
  }

  return gPreferences;
}

//May not be needed, but just incase.
function wikipedia_setupDocument(aDoc)
{
  wikipedia_Document = aDoc;
  wikipedia_stringBundle = aDoc.getElementById("wikipediastrings");
}

//Make sure wikipedia loads on start-up of browser.
window.addEventListener('load', wikipedia.init, false);